@@ -10,6 +10,14 @@ import cloudpickle
 import dask.dataframe as dd
 import numpy as np
 import pandas as pd
+from woodwork.logical_types import (
+    Age,
+    AgeNullable,
+    Boolean,
+    BooleanNullable,
+    Integer,
+    IntegerNullable
+)
 
 from featuretools.computational_backends.feature_set import FeatureSet
 from featuretools.computational_backends.feature_set_calculator import (
@@ -339,7 +347,7 @@ def calculate_chunk(cutoff_time, chunk_size, feature_set, entityset, approximate
                     pass_columns, progress_bar=None, progress_callback=None, include_cutoff_time=True, schema=None):
 
     if not isinstance(feature_set, FeatureSet):
-        feature_set = cloudpickle.loads(feature_set)
+        feature_set = cloudpickle.loads(feature_set)   # pragma: no cover
 
     feature_matrix = []
     if no_unapproximated_aggs and approximate is not None:
@@ -375,7 +383,7 @@ def calculate_chunk(cutoff_time, chunk_size, feature_set, entityset, approximate
 
     else:
         if schema:
-            cutoff_time.ww.init_with_full_schema(schema=schema)
+            cutoff_time.ww.init_with_full_schema(schema=schema)  # pragma: no cover
         for _, group in cutoff_time.groupby(cutoff_df_time_col):
             # if approximating, calculate the approximate features
             if approximate is not None:
@@ -636,7 +644,7 @@ def parallel_calculate_chunks(cutoff_time, chunk_size, feature_set, approximate,
 
         chunks = [df for _, df in chunks]
 
-        if len(chunks) < num_workers:
+        if len(chunks) < num_workers:  # pragma: no cover
             chunk_warning = "Fewer chunks ({}), than workers ({}) consider reducing the chunk size"
             warning_string = chunk_warning.format(len(chunks), num_workers)
             progress_bar.write(warning_string)
@@ -690,7 +698,7 @@ def parallel_calculate_chunks(cutoff_time, chunk_size, feature_set, approximate,
             client.close()
 
         if 'cluster' not in dask_kwargs and cluster is not None:
-            cluster.close()
+            cluster.close()  # pragma: no cover
 
     ww_init_kwargs = get_ww_types_from_features(feature_set.target_features, entityset, pass_columns, cutoff_time)
     feature_matrix = init_ww_and_concat_fm(feature_matrix, ww_init_kwargs)
@@ -759,7 +767,27 @@ def update_progress_callback_parameters(progress_bar, previous_progress):
 
 
 def init_ww_and_concat_fm(feature_matrix, ww_init_kwargs):
+    cols_to_check = {col for col, ltype in
+                     ww_init_kwargs["logical_types"].items()
+                     if isinstance(ltype, (Age, Boolean, Integer))}
+    replacement_type = {
+        "age": AgeNullable(),
+        "boolean": BooleanNullable(),
+        "integer": IntegerNullable(),
+    }
     for fm in feature_matrix:
+        updated_cols = set()
+        for col in cols_to_check:
+            # Only convert types for pandas if null values are present
+            # Always convert for Dask/Koalas to avoid pulling data into memory for null check
+            is_pandas_df_with_null = isinstance(fm, pd.DataFrame) and fm[col].isnull().any()
+            is_dask_df = isinstance(fm, dd.DataFrame)
+            is_koalas_df = is_instance(fm, ks, 'DataFrame')
+            if is_pandas_df_with_null or is_dask_df or is_koalas_df:
+                current_type = ww_init_kwargs["logical_types"][col].type_string
+                ww_init_kwargs["logical_types"][col] = replacement_type[current_type]
+                updated_cols.add(col)
+        cols_to_check = cols_to_check - updated_cols
         fm.ww.init(**ww_init_kwargs)
 
     if any(isinstance(fm, dd.DataFrame) for fm in feature_matrix):
