@@ -671,6 +671,9 @@ class EntitySet(object):
         self._add_references_to_metadata(dataframe)
         return self
 
+    def __setitem__(self, key, value):
+        self.add_dataframe(dataframe=value, dataframe_name=key)
+
     def normalize_dataframe(self, base_dataframe_name, new_dataframe_name, index,
                             additional_columns=None, copy_columns=None,
                             make_time_index=None,
@@ -798,6 +801,7 @@ class EntitySet(object):
             [col for col in additional_columns] +\
             [col for col in copy_columns]
 
+        new_dataframe = new_dataframe.dropna(subset=[index])
         new_dataframe2 = new_dataframe. \
             drop_duplicates(index, keep='first')[selected_columns]
 
@@ -1150,7 +1154,7 @@ class EntitySet(object):
         ww_schemas = state.pop(WW_SCHEMA_KEY)
         for df_name, df in state.get('dataframe_dict', {}).items():
             if ww_schemas[df_name] is not None:
-                df.ww.init_with_full_schema(schema=ww_schemas[df_name], validate=False)
+                df.ww.init(schema=ww_schemas[df_name], validate=False)
         self.__dict__.update(state)
 
     # ###########################################################################
