@@ -4,6 +4,7 @@ from featuretools.feature_base import (
     GroupByTransformFeature,
     TransformFeature
 )
+from featuretools.utils.gen_utils import camel_and_title_to_snake
 
 
 def _categorize_features(features):
@@ -53,7 +54,7 @@ def get_unused_primitives(specified, used):
     if not specified:
         return []
     specified = {
-        primitive.lower() if isinstance(primitive, str) else primitive.name
+        camel_and_title_to_snake(primitive) if isinstance(primitive, str) else primitive.name
         for primitive in specified
     }
     return sorted(list(specified.difference(used)))
