@@ -13,6 +13,14 @@ import pytest
 from dask import dataframe as dd
 from tqdm import tqdm
 from woodwork.column_schema import ColumnSchema
+from woodwork.logical_types import (
+    Age,
+    AgeNullable,
+    Boolean,
+    BooleanNullable,
+    Integer,
+    IntegerNullable
+)
 
 import featuretools as ft
 from featuretools import EntitySet, Timedelta, calculate_feature_matrix, dfs
@@ -1238,7 +1246,7 @@ class TestCreateClientAndCluster(object):
                             get_mock_client_cluster)
         try:
             cpus = len(psutil.Process().cpu_affinity())
-        except AttributeError:
+        except AttributeError:  # pragma: no cover
             cpus = psutil.cpu_count()
 
         # jobs < tasks case
@@ -1305,7 +1313,7 @@ def test_parallel_failure_raises_correct_error(pd_es):
                                  approximate='1 hour')
 
 
-def test_warning_not_enough_chunks(pd_es, capsys, three_worker_scheduler):
+def test_warning_not_enough_chunks(pd_es, capsys, three_worker_scheduler):  # pragma: no cover
     property_feature = IdentityFeature(pd_es['log'].ww['value']) > 10
 
     dkwargs = {'cluster': three_worker_scheduler['address']}
@@ -1323,7 +1331,7 @@ def test_warning_not_enough_chunks(pd_es, capsys, three_worker_scheduler):
 def test_n_jobs():
     try:
         cpus = len(psutil.Process().cpu_affinity())
-    except AttributeError:
+    except AttributeError:  # pragma: no cover
         cpus = psutil.cpu_count()
 
     assert n_jobs_to_workers(1) == 1
@@ -1739,15 +1747,12 @@ def test_closes_tqdm(es):
 
     assert len(tqdm._instances) == 0
 
-    try:
+    match = "This primitive has errored"
+    with pytest.raises(RuntimeError, match=match):
         calculate_feature_matrix([value, error_feature],
                                  es,
                                  verbose=True)
-        assert False
-    except RuntimeError as e:
-        assert e.args[0] == "This primitive has errored"
-    finally:
-        assert len(tqdm._instances) == 0
+    assert len(tqdm._instances) == 0
 
 
 def test_approximate_with_single_cutoff_warns(pd_es):
@@ -1868,3 +1873,24 @@ def test_cfm_with_invalid_time_index(es):
     match += "which differs from other entityset time indexes"
     with pytest.raises(TypeError, match=match):
         calculate_feature_matrix(features=features, entityset=es)
+
+
+def test_cfm_introduces_nan_values_in_direct_feats(es):
+    es["customers"].ww.set_types(logical_types={"age": "Age",
+                                                "engagement_level": "Integer"})
+    age_feat = ft.Feature(es["customers"].ww["age"])
+    engagement_feat = ft.Feature(es["customers"].ww["engagement_level"])
+    loves_ice_cream_feat = ft.Feature(es["customers"].ww["loves_ice_cream"])
+    features = [age_feat, engagement_feat, loves_ice_cream_feat]
+    fm = calculate_feature_matrix(features=features,
+                                  entityset=es,
+                                  cutoff_time=pd.Timestamp("2010-04-08 04:00"),
+                                  instance_ids=[1])
+
+    assert isinstance(es["customers"].ww.logical_types["age"], Age)
+    assert isinstance(es["customers"].ww.logical_types["engagement_level"], Integer)
+    assert isinstance(es["customers"].ww.logical_types["loves_ice_cream"], Boolean)
+
+    assert isinstance(fm.ww.logical_types["age"], AgeNullable)
+    assert isinstance(fm.ww.logical_types["engagement_level"], IntegerNullable)
+    assert isinstance(fm.ww.logical_types["loves_ice_cream"], BooleanNullable)
