@@ -1,4 +1,5 @@
 import copy
+import re
 
 import pandas as pd
 import pytest
@@ -77,17 +78,6 @@ def test_makes_agg_features_from_mixed_str(es):
     assert (feature_with_name(features, 'COUNT(log)'))
 
 
-def test_case_insensitive(es):
-    dfs_obj = DeepFeatureSynthesis(target_dataframe_name='sessions',
-                                   entityset=es,
-                                   agg_primitives=['MiN'],
-                                   trans_primitives=['AbsOlute'])
-
-    features = dfs_obj.build_features()
-    assert (feature_with_name(features, 'MIN(log.value)'))
-    assert (feature_with_name(features, 'ABSOLUTE(MIN(log.value_many_nans))'))
-
-
 def test_makes_agg_features(es):
     dfs_obj = DeepFeatureSynthesis(target_dataframe_name='sessions',
                                    entityset=es,
@@ -408,7 +398,9 @@ def test_make_groupby_features_with_agg(pd_es):
 
 
 def test_bad_groupby_feature(es):
-    msg = "Unknown transform primitive max"
+    msg = re.escape("Unknown groupby transform primitive max. "
+                    "Call ft.primitives.list_primitives() to get "
+                    "a list of available primitives")
     with pytest.raises(ValueError, match=msg):
         DeepFeatureSynthesis(target_dataframe_name='customers',
                              entityset=es,
@@ -426,25 +418,41 @@ def test_bad_groupby_feature(es):
         RollingSTD,
     ]
 )
-def test_make_rolling_features(rolling_primitive, pd_es):
-    rolling_primitive_obj = rolling_primitive(window_length=7, gap=3, min_periods=5)
+@pytest.mark.parametrize(
+    "window_length, gap",
+    [
+        (7, 3),
+        ('7d', '3d'),
+    ]
+)
+def test_make_rolling_features(window_length, gap, rolling_primitive, pd_es):
+    rolling_primitive_obj = rolling_primitive(window_length=window_length,
+                                              gap=gap,
+                                              min_periods=5)
     dfs_obj = DeepFeatureSynthesis(target_dataframe_name='log',
                                    entityset=pd_es,
                                    agg_primitives=[],
                                    trans_primitives=[rolling_primitive_obj])
     features = dfs_obj.build_features()
-    rolling_transform_name = f"{rolling_primitive.name.upper()}(datetime, value_many_nans, window_length=7, gap=3, min_periods=5)"
+    rolling_transform_name = f"{rolling_primitive.name.upper()}(datetime, value_many_nans, window_length={window_length}, gap={gap}, min_periods=5)"
     assert feature_with_name(features, rolling_transform_name)
 
 
-def test_make_rolling_count_off_datetime_feature(pd_es):
-    rolling_count = RollingCount(window_length=5, min_periods=3)
+@pytest.mark.parametrize(
+    "window_length, gap",
+    [
+        (7, 3),
+        ('7d', '3d'),
+    ]
+)
+def test_make_rolling_count_off_datetime_feature(window_length, gap, pd_es):
+    rolling_count = RollingCount(window_length=window_length, min_periods=gap)
     dfs_obj = DeepFeatureSynthesis(target_dataframe_name='log',
                                    entityset=pd_es,
                                    agg_primitives=[],
                                    trans_primitives=[rolling_count])
     features = dfs_obj.build_features()
-    rolling_transform_name = u"ROLLING_COUNT(datetime, window_length=5, min_periods=3)"
+    rolling_transform_name = f"ROLLING_COUNT(datetime, window_length={window_length}, min_periods={gap})"
     assert feature_with_name(features, rolling_transform_name)
 
 
@@ -925,7 +933,7 @@ def test_return_types(es):
 
 def test_checks_primitives_correct_type(es):
     error_text = "Primitive <class \\'featuretools\\.primitives\\.standard\\."\
-                 "transform_primitive\\.Hour\\'> in agg_primitives is not an "\
+                 "datetime_transform_primitives\\.Hour\\'> in agg_primitives is not an "\
                  "aggregation primitive"
     with pytest.raises(ValueError, match=error_text):
         DeepFeatureSynthesis(target_dataframe_name="sessions",
@@ -934,8 +942,8 @@ def test_checks_primitives_correct_type(es):
                              trans_primitives=[])
 
     error_text = "Primitive <class \\'featuretools\\.primitives\\.standard\\."\
-                 "aggregation_primitives\\.Sum\\'> in trans_primitives or "\
-                 "groupby_trans_primitives is not a transform primitive"
+                 "aggregation_primitives\\.Sum\\'> in trans_primitives "\
+                 "is not a transform primitive"
     with pytest.raises(ValueError, match=error_text):
         DeepFeatureSynthesis(target_dataframe_name="sessions",
                              entityset=es,
