@@ -1,5 +1,6 @@
 import importlib
 import logging
+import re
 import sys
 import warnings
 from enum import Enum
@@ -132,6 +133,12 @@ def is_instance(obj, modules, classnames):
     return isinstance(obj, to_check)
 
 
+def camel_and_title_to_snake(name):
+    name = re.sub(r"(\d+)", r"_\1", name).strip('_')
+    name = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', name)
+    return re.sub('([a-z0-9])([A-Z])', r'\1_\2', name).lower()
+
+
 class Library(Enum):
     PANDAS = 'pandas'
     DASK = 'Dask'
