@@ -6,7 +6,7 @@ def replace_tuple_columns(pdf):
     new_df = pd.DataFrame()
     for c in pdf.columns:
         if isinstance(pdf[c].iloc[0], tuple):
-            new_df[c] = pdf[c].map(lambda x: list(x))
+            new_df[c] = pdf[c].map(lambda x: list(x) if isinstance(x, tuple) else x)
         else:
             new_df[c] = pdf[c]
     return new_df
@@ -16,7 +16,7 @@ def replace_nan_with_flag(pdf, flag=-1):
     new_df = pd.DataFrame()
     for c in pdf.columns:
         if isinstance(pdf[c].iloc[0], list):
-            new_df[c] = pdf[c].map(lambda l: [flag if np.isnan(x) else x for x in l])
+            new_df[c] = pdf[c].map(lambda l: [flag if np.isnan(x) else x for x in l] if isinstance(l, (tuple, list)) else l)
         else:
             new_df[c] = pdf[c]
     return new_df
@@ -27,7 +27,7 @@ def replace_categorical_columns(pdf):
     for c in pdf.columns:
         col = pdf[c]
         if col.dtype.name == 'category':
-            new_df[c] = col.astype(col.dtype.categories.dtype)
+            new_df[c] = col.astype('string')
         else:
             new_df[c] = pdf[c]
     return new_df
